# The contents of this file are subject to the Common Public Attribution License
# Version 1.0 (the “License”); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
# https://myou.dev/licenses/LICENSE-CPAL. The License is based on the Mozilla
# Public License Version 1.1 but Sections 14 and 15 have been added to cover use
# of software over a computer network and provide for limited attribution for
# the Original Developer. In addition, Exhibit A has been modified to be
# consistent with Exhibit B.
#
# Software distributed under the License is distributed on an “AS IS” basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
# the specific language governing rights and limitations under the License.
#
# The Original Code is Myou Engine.
#
# the Original Developer is the Initial Developer.
#
# The Initial Developer of the Original Code is the Myou Engine developers.
# All portions of the code written by the Myou Engine developers are Copyright
# (c) 2024. All Rights Reserved.
#
# Alternatively, the contents of this file may be used under the terms of the
# GNU Affero General Public License version 3 (the [AGPL-3] License), in which
# case the provisions of [AGPL-3] License are applicable instead of those above.
#
# If you wish to allow use of your version of this file only under the terms of
# the [AGPL-3] License and not to allow others to use your version of this file
# under the CPAL, indicate your decision by deleting the provisions above and
# replace them with the notice and other provisions required by the [AGPL-3]
# License. If you do not delete the provisions above, a recipient may use your
# version of this file under either the CPAL or the [AGPL-3] License.

import ../types
import vmath except Quat

when defined(nimdoc):
    type TYPES* = LightType | Light

# Forward declarations and ob type methods
proc newLight*(engine: MyouEngine, name: string="",
        scene: Scene = nil,
        light_type: LightType = PointLight,
        color: Vec3 = vec3(1, 1, 1),
        energy: float32 = 1,
        cutoff_distance: float32 = 0,
        diffuse_factor: float32 = 1,
        specular_factor: float32 = 1,
        light_radius: float32 = 0.01,
        spot_size: float32 = 1.3,
        spot_blend: float32 = 0.15,
    ): Light
proc instance_physics*(this: Light)

method is_light*(self: GameObject): bool {.base.} =
    return false
method get_light*(self: GameObject): Light {.base.} =
    return nil
method is_light*(this: Light): bool =
    return true
method get_light*(this: Light): Light =
    return this
# End forward declarations and ob type methods

import ./gameobject
import ../scene

proc newLight*(engine: MyouEngine, name: string="",
        scene: Scene = nil,
        light_type: LightType = PointLight,
        color: Vec3 = vec3(1, 1, 1),
        energy: float32 = 1,
        cutoff_distance: float32 = 0,
        diffuse_factor: float32 = 1,
        specular_factor: float32 = 1,
        light_radius: float32 = 0.01,
        spot_size: float32 = 1.3,
        spot_blend: float32 = 0.15,
    ): Light =
    var this = new Light
    discard procCall(this.GameObject.initGameObject(engine, name))
    this.otype = TLight
    this.light_type = light_type
    this.color = color
    this.energy = energy
    this.light_radius = light_radius
    this.spot_size = spot_size
    this.spot_blend = spot_blend
    this.cutoff_distance = cutoff_distance
    this.diffuse_factor = diffuse_factor
    this.specular_factor = specular_factor
    if scene != nil:
        scene.add_object(this, name=name)
    return this

proc instance_physics*(this: Light) =
    discard

